/*
   This scene contains two lighted cubes.
   One cube has "sphere like" normal vectors at each vertex,
   and the other cube has "cube like" normal vectors at each vertex.
*/
import java.awt.Color;

public class LightedCubes
{
   public static void main(String[] args)
   {
      // Create a Material object (brass).
      Material mtrl = new Material();
      mtrl.ambient[0] = 0.33f;
      mtrl.ambient[1] = 0.22f;
      mtrl.ambient[2] = 0.03f;
      mtrl.ambient[3] = 1.00f;
      mtrl.diffuse[0] = 0.78f;
      mtrl.diffuse[1] = 0.57f;
      mtrl.diffuse[2] = 0.11f;
      mtrl.diffuse[3] = 1.00f;
      mtrl.specular[0] = 0.99f;
      mtrl.specular[1] = 0.91f;
      mtrl.specular[2] = 0.81f;
      mtrl.specular[3] = 1.00f;
      mtrl.s = 27.8f;

      // Here are the vertices of the standard cube.
      Vertex[] v = { new Vertex( 1, -1,  1),
                     new Vertex( 1, -1, -1),
                     new Vertex(-1, -1, -1),
                     new Vertex(-1, -1,  1),
                     new Vertex( 1,  1,  1),
                     new Vertex( 1,  1, -1),
                     new Vertex(-1,  1, -1),
                     new Vertex(-1,  1,  1) };

      // Give each vertex a "sphere like" normal vector.
      double sqrt3 = Math.sqrt(3.0);
      v[0].n = new Vector( 1/sqrt3, -1/sqrt3,  1/sqrt3);
      v[1].n = new Vector( 1/sqrt3, -1/sqrt3, -1/sqrt3);
      v[2].n = new Vector(-1/sqrt3, -1/sqrt3, -1/sqrt3);
      v[3].n = new Vector(-1/sqrt3, -1/sqrt3,  1/sqrt3);
      v[4].n = new Vector( 1/sqrt3,  1/sqrt3,  1/sqrt3);
      v[5].n = new Vector( 1/sqrt3,  1/sqrt3, -1/sqrt3);
      v[6].n = new Vector(-1/sqrt3,  1/sqrt3, -1/sqrt3);
      v[7].n = new Vector(-1/sqrt3,  1/sqrt3,  1/sqrt3);

      // Now make 12 triangles.
      Triangle[] t1 = { new Triangle(v[0], v[1], v[2]),  // bottom
                        new Triangle(v[2], v[3], v[0]),

                        new Triangle(v[4], v[7], v[6]),  // top
                        new Triangle(v[6], v[5], v[4]),

                        new Triangle(v[0], v[4], v[7]),  // front
                        new Triangle(v[7], v[3], v[0]),

                        new Triangle(v[1], v[2], v[6]),  // back
                        new Triangle(v[6], v[5], v[1]),

                        new Triangle(v[0], v[1], v[5]),  // right
                        new Triangle(v[5], v[4], v[0]),

                        new Triangle(v[2], v[3], v[7]),  // left
                        new Triangle(v[7], v[6], v[2])};

      // Create an empty Model object.
      Model m1 = new Model();
      m1.doLighting = true;
      m1.doSmoothLighting = true;

      m1.modelTranslate(2, 1, 0);

      // Put the (sphere like) triangles in the model.
      for (int i = 0; i < t1.length; i++)
      {
         t1[i].mtrl = mtrl;
         m1.triangleList.add( t1[i] );
      }

      // Now create 12 triangles with "cube like" normal vectors.
      Triangle[] t2 = new Triangle[12];
      Vector n; // used for the normal vectors
      int i = 0;
      // bottom face
      n = new Vector(0.0, -1.0, 0.0);
      v[0].n = v[1].n = v[2].n = v[3].n = n;
      t2[i++] = new Triangle(v[0], v[1], v[2]);
      t2[i++] = new Triangle(v[2], v[3], v[0]);
      // top face
      n = new Vector(0.0, 1.0, 0.0);
      v[4].n = v[5].n = v[6].n = v[7].n = n;
      t2[i++] = new Triangle(v[4], v[7], v[6]);
      t2[i++] = new Triangle(v[6], v[5], v[4]);
      // front face
      n = new Vector(0.0, 0.0, 1.0);
      v[0].n = v[3].n = v[4].n = v[7].n = n;
      t2[i++] = new Triangle(v[0], v[4], v[7]);
      t2[i++] = new Triangle(v[7], v[3], v[0]);
      // back face
      n = new Vector(0.0, 0.0, -1.0);
      v[1].n = v[2].n = v[5].n = v[6].n = n;
      t2[i++] = new Triangle(v[1], v[2], v[6]);
      t2[i++] = new Triangle(v[6], v[5], v[1]);
      // right face
      n = new Vector(1.0, 0.0, 0.0);
      v[0].n = v[1].n = v[4].n = v[5].n = n;
      t2[i++] = new Triangle(v[0], v[1], v[5]);
      t2[i++] = new Triangle(v[5], v[4], v[0]);
      // left face
      n = new Vector(-1.0, 0.0, 0.0);
      v[2].n = v[3].n = v[6].n = v[7].n = n;
      t2[i++] = new Triangle(v[2], v[3], v[7]);
      t2[i++] = new Triangle(v[7], v[6], v[2]);

      // Create an empty Model object.
      Model m2 = new Model();
      m2.doLighting = true;
      m2.doSmoothLighting = true;

      m2.modelTranslate(-2, 1, 0);

      // Put the (cube like) triangles in this model
      for (int j = 0; j < t2.length; j++)
      {
         t2[j].mtrl = mtrl;
         m2.triangleList.add( t2[j] );
      }

      // Give the scene a floor object.
      Model floor = new PanelModel(-5, 5, -5, 5, Color.white, Color.black);
      floor.doLighting = false;
      floor.modelTranslate(0, 0, 0);

      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the models to the Scene.
      scene.modelList.add(m1);
      scene.modelList.add(m2);
      scene.modelList.add(floor);


      // Create a Light object for the scene.
      Light light = new Light();
      light.pos[0] = 0.0f;
      light.pos[1] = 1.0f;
      light.pos[2] = 10.0f;
      light.pos[3] = 1.0f;
      light.ambient[0] = 1.0f;
      light.ambient[1] = 1.0f;
      light.ambient[2] = 1.0f;
      light.ambient[3] = 1.0f;
      light.diffuse[0] = 1.0f;
      light.diffuse[1] = 1.0f;
      light.diffuse[2] = 1.0f;
      light.diffuse[3] = 1.0f;
      light.specular[0] = 1.0f;
      light.specular[1] = 1.0f;
      light.specular[2] = 1.0f;
      light.specular[3] = 1.0f;
      // Add the light to the scene.
      scene.light = light;


      // Give the Scene a view matrix.
      scene.view2Identity();

      // Give the scene a projection matrix.
      scene.proj2Identity();

      // Set up the scene's view frustum.
      double fov    = 60.0;
      double aspect = 1.0;
      double near   = 1.0;
      double far    = 20.0;
      scene.projPerspective(fov, aspect, near, far);

      // Set up the scene's "camera".
      double eyex=0,    eyey=4,    eyez=4;
      double centerx=0, centery=0, centerz=0;
      double upx=0,     upy=1,     upz=0;
      scene.viewLookAt(eyex,    eyey,    eyez,
                       centerx, centery, centerz,
                       upx,     upy,     upz);

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(java.awt.Color.darkGray);
      // Render our scene into the frame buffer.
      SceneRender.renderScene(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("LightedCubes_Frame%02d.ppm", 0) );

      // Spin the models.
      for (int j = 1; j <= 90; j++)
      {
         // Rotate the scene by 10 degrees.
         m1.modelRotate(1, 0, 1, 0);
         m2.modelRotate(1, 0, 1, 0);

         // Render again.
         fb.clearFB(java.awt.Color.darkGray);
         SceneRender.renderScene(scene, fb);
         fb.dumpFB2File(String.format("LightedCubes_Frame%02d.ppm", j));
      }
   }
}//LightedCubes